#version 330

uniform sampler2D sourceTexture;
uniform int filterStep;
uniform float widthOffset;
uniform float heightOffset;

in vec2 fragmentUV;

out vec4 outputColor;

const float gaussNormFactor = 6;
//const float gauss[7] = float [] (0.0151618, 0.0283261, 0.0412142, 0.0467018, 0.0412142, 0.0283261, 0.0151618); sigma = ??
const float gauss[7] = float [] (0.0290, 0.1038, 0.2232, 0.2880, 0.2232, 0.1038, 0.0290); // sigma = 1.4
//const float gauss[5] = float [] (0.1038, 0.2232, 0.2880, 0.2232, 0.1038); // sigma = 1.4
//const float gauss[7] = float [] (0.0702, 0.1311, 0.1907, 0.2161, 0.1907, 0.1311, 0.0702); // sigma = 2
//const float gauss[7] = float [] (0.0887, 0.1369, 0.1776, 0.1937, 0.1776, 0.1369, 0.0887); // sigma = 2.4
//const float gauss[7] = float [] (0.1063, 0.1403, 0.1658, 0.1752, 0.1658, 0.1403, 0.1063); // sigma = 3
const int s = 7; // kernel size
const int hs = s / 2;
const float k = 1.0f; // factor for pixel step (filter kernel stretch)

const float minorFactor = 0.3125f; // 5/16
const float mayorFactor = 0.375f; // 6/16

void main() {
	vec2 pxStep = 1.0 / textureSize(sourceTexture, 0);
	vec3 pxColor = vec3(0);

	if(filterStep == 0) {
		for (int i = 0; i < s - 1; ++i) {
			pxColor += texture(sourceTexture, vec2(fragmentUV.x + (i - hs) * pxStep.x * k, fragmentUV.y)).rgb * gauss[i];
		}
	} else {
		for (int i = 0; i < s - 1; ++i) {
			pxColor += texture(sourceTexture, vec2(fragmentUV.x, fragmentUV.y + (i - hs) * pxStep.y * k)).rgb * gauss[i];
		}
	}

	outputColor = vec4(pxColor, 1);
	
	/*
	vec2 firstOffset;
	vec2 secondOffset;

	if(filterStep == 0) {
		firstOffset = vec2(fragmentUV.x - widthOffset, fragmentUV.y);
		secondOffset = vec2(fragmentUV.x + widthOffset, fragmentUV.y);
	} else {
		firstOffset = vec2(fragmentUV.x, fragmentUV.y - heightOffset);
		secondOffset = vec2(fragmentUV.x, fragmentUV.y + heightOffset);
	}

	outputColor = vec4(
		texture(sourceTexture, firstOffset).rgb*minorFactor +
		texture(sourceTexture, fragmentUV).rgb*mayorFactor +
		texture(sourceTexture, secondOffset).rgb*minorFactor, 1);

	*/
}
