#version 330

uniform sampler2D hdrTex;

uniform int hdrTexLod;
uniform float key;
uniform float YWhite;
uniform float sat;

const vec3 luminance = vec3(0.2125f, 0.7154f, 0.0721f);

const mat3 RGB2XYZ = mat3(0.4124, 0.2126, 0.0193, 0.3576, 0.7152, 0.1192, 0.1805, 0.0722, 0.9505);
const mat3 XYZ2RGB = mat3(3.2405, -0.9693, 0.0556, -1.5371, 1.8760, -0.2040, -0.4985, 0.0416, 1.0572);

in vec2 fragmentUV;

out vec3 outputColor;

vec3 reinhardToneMapping(vec3 hdrColor, float logAvgLum);
vec3 reinhardModToneMapping(vec3 hdrColor, float logAvgLum);
vec3 RGB2Yxy(vec3 rgb);
vec3 Yxy2RGB(vec3 Yxy);

vec3 toneMapping(vec3 rgb, float logAvgLum) {
	vec3 XYZ = RGB2XYZ*rgb;
	float Y = (key / logAvgLum) * XYZ. y;
	float Yd = (Y * (1.0 + Y / (YWhite * YWhite) ) ) / (1.0 + Y);

	return pow(rgb / XYZ.y , vec3 ( sat , sat , sat ) ) * Yd;
}

//Computes the low dynamic range image out of the high dynamic range data using the modified Reinhard's operator
void main() {
	vec4 meanColor = texture(hdrTex, fragmentUV, hdrTexLod);
	float logAvgLum = exp(meanColor.a);

	outputColor = texture(hdrTex, fragmentUV).rgb;

	outputColor = toneMapping(outputColor, logAvgLum);
}

vec3 reinhardToneMapping(vec3 hdrColor, float logAvgLum) {
	vec3 lScaled = key*hdrColor/logAvgLum;

	return lScaled/(vec3(1,1,1) + lScaled);
}

vec3 reinhardModToneMapping(vec3 hdrColor, float logAvgLum) {
	vec3 Yxy = RGB2Yxy(hdrColor);

	float lScaled = Yxy.x*key/logAvgLum;

	Yxy.x = (lScaled*(1 + (lScaled/(YWhite*YWhite)) ))/(1 + lScaled);
	return Yxy2RGB(Yxy);
}

vec3 RGB2Yxy(vec3 rgb) {
	vec3 xyz = RGB2XYZ * rgb;
	return  vec3(xyz.y, xyz.x/(xyz.x+xyz.y+xyz.z), xyz.y/(xyz.x+xyz.y+xyz.z));
}

vec3 Yxy2RGB(vec3 Yxy) {
	vec3 xyz = vec3(Yxy.y*(Yxy.x/Yxy.z), Yxy.x, (1 - Yxy.y - Yxy.z)*(Yxy.x/Yxy.z));

	return XYZ2RGB * xyz;
}